@@ -5,6 +5,8 @@ import (
 
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1"
 	"github.com/k0sproject/k0sctl/pkg/apis/k0sctl.k0sproject.io/v1beta1/cluster"
+	"github.com/k0sproject/rig/exec"
+	"github.com/sirupsen/logrus"
 	log "github.com/sirupsen/logrus"
 )
 
@@ -66,6 +68,9 @@ func (p *DownloadK0s) downloadK0s(h *cluster.Host) error {
 	if err := h.Configurer.DownloadK0s(h, tmp, p.Config.Spec.K0s.Version, h.Metadata.Arch); err != nil {
 		return err
 	}
+	if err := h.Execf(`chmod +x "%s"`, tmp, exec.Sudo(h)); err != nil {
+		logrus.Warnf("%s: failed to chmod k0s temp binary: %v", h, err.Error())
+	}
 
 	h.Metadata.K0sBinaryTempFile = tmp
 
