@@ -48,17 +48,6 @@ func (p *ResetControllers) ShouldRun() bool {
 	return len(p.hosts) > 0
 }
 
-// CleanUp cleans up the environment override files on hosts
-func (p *ResetControllers) CleanUp() {
-	for _, h := range p.hosts {
-		if len(h.Environment) > 0 {
-			if err := h.Configurer.CleanupServiceEnvironment(h, h.K0sServiceName()); err != nil {
-				log.Warnf("%s: failed to clean up service environment: %s", h, err.Error())
-			}
-		}
-	}
-}
-
 // Run the phase
 func (p *ResetControllers) Run() error {
 	for _, h := range p.hosts {
@@ -98,13 +87,18 @@ func (p *ResetControllers) Run() error {
 			log.Debugf("%s: stopping k0s completed", h)
 		}
 
-		log.Debugf("%s: leaving etcd...", h)
 		if !p.NoLeave {
-			if err := p.leader.LeaveEtcd(h); err != nil {
+			log.Debugf("%s: leaving etcd...", h)
+
+			etcdAddress := h.SSH.Address
+			if h.PrivateAddress != "" {
+				etcdAddress = h.PrivateAddress
+			}
+			if err := h.Exec(h.Configurer.K0sCmdf("etcd leave --peer-address %s --datadir %s", etcdAddress, h.K0sDataDir()), exec.Sudo(h)); err != nil {
 				log.Warnf("%s: failed to leave etcd: %s", h, err.Error())
 			}
+			log.Debugf("%s: leaving etcd completed", h)
 		}
-		log.Debugf("%s: leaving etcd completed", h)
 
 		log.Debugf("%s: resetting k0s...", h)
 		out, err := h.ExecOutput(h.Configurer.K0sCmdf("reset --data-dir=%s", h.K0sDataDir()), exec.Sudo(h))
@@ -120,6 +114,12 @@ func (p *ResetControllers) Run() error {
 		}
 		log.Debugf("%s: removing config completed", h)
 
+		if len(h.Environment) > 0 {
+			if err := h.Configurer.CleanupServiceEnvironment(h, h.K0sServiceName()); err != nil {
+				log.Warnf("%s: failed to clean up service environment: %s", h, err.Error())
+			}
+		}
+
 		log.Infof("%s: reset", h)
 	}
 	return nil
