@@ -24,19 +24,24 @@ func (p *Restore) Title() string {
 
 // ShouldRun is true when there path to backup file
 func (p *Restore) ShouldRun() bool {
-	return p.RestoreFrom != "" && p.leader.Metadata.K0sRunningVersion == nil
+	return p.RestoreFrom != "" && p.leader.Metadata.K0sRunningVersion == nil && !p.leader.Reset
 }
 
 // Prepare the phase
 func (p *Restore) Prepare(config *v1beta1.Cluster) error {
-	log.Tracef("restore from: %s", p.RestoreFrom)
 	p.Config = config
-	p.leader = p.Config.Spec.K0sLeader()
 
-	if p.RestoreFrom != "" && p.leader.Exec(p.leader.Configurer.K0sCmdf("restore --help"), exec.Sudo(p.leader)) != nil {
+	if p.RestoreFrom == "" {
+		return nil
+	}
+
+	// defined in backup.go
+	if !backupSinceVersion.Check(p.Config.Spec.K0s.Version) {
 		return fmt.Errorf("the version of k0s on the host does not support restoring backups")
 	}
 
+	p.leader = p.Config.Spec.K0sLeader()
+
 	log.Tracef("restore leader: %s", p.leader)
 	log.Tracef("restore leader state: %+v", p.leader.Metadata)
 	return nil
