@@ -3,7 +3,7 @@ package cluster
 import (
 	"fmt"
 	gos "os"
-	"path/filepath"
+	gopath "path"
 	"regexp"
 	"strconv"
 	"strings"
@@ -11,9 +11,9 @@ import (
 
 	"github.com/alessio/shellescape"
 	"github.com/creasty/defaults"
-	validation "github.com/go-ozzo/ozzo-validation/v4"
-	"github.com/go-ozzo/ozzo-validation/v4/is"
 	"github.com/go-playground/validator/v10"
+	"github.com/jellydator/validation"
+	"github.com/jellydator/validation/is"
 	"github.com/k0sproject/rig"
 	"github.com/k0sproject/rig/exec"
 	"github.com/k0sproject/rig/os"
@@ -168,6 +168,7 @@ type configurer interface {
 	K0sctlLockFilePath(os.Host) string
 	UpsertFile(os.Host, string, string) error
 	MachineID(os.Host) (string, error)
+	SetPath(string, string)
 }
 
 // HostMetadata resolved metadata for host
@@ -175,12 +176,18 @@ type HostMetadata struct {
 	K0sBinaryVersion  *version.Version
 	K0sBinaryTempFile string
 	K0sRunningVersion *version.Version
+	K0sInstalled      bool
+	K0sExistingConfig string
+	K0sNewConfig      string
+	K0sJoinToken      string
+	K0sJoinTokenID    string
 	Arch              string
 	IsK0sLeader       bool
 	Hostname          string
 	Ready             bool
 	NeedsUpgrade      bool
 	MachineID         string
+	DryRunFakeLeader  bool
 }
 
 // UnmarshalYAML sets in some sane defaults when unmarshaling the data from yaml
@@ -203,14 +210,9 @@ func (h *Host) UnmarshalYAML(unmarshal func(interface{}) error) error {
 
 // Address returns an address for the host
 func (h *Host) Address() string {
-	if h.SSH != nil {
-		return h.SSH.Address
+	if addr := h.Connection.Address(); addr != "" {
+		return addr
 	}
-
-	if h.WinRM != nil {
-		return h.WinRM.Address
-	}
-
 	return "127.0.0.1"
 }
 
@@ -369,13 +371,17 @@ func (h *Host) K0sServiceName() string {
 	}
 }
 
+func (h *Host) k0sBinaryPathDir() string {
+	return gopath.Dir(h.Configurer.K0sBinaryPath())
+}
+
 // InstallK0sBinary installs the k0s binary from the provided file path to K0sBinaryPath
 func (h *Host) InstallK0sBinary(path string) error {
 	if !h.Configurer.FileExist(h, path) {
 		return fmt.Errorf("k0s binary tempfile not found")
 	}
 
-	dir := filepath.Dir(h.Configurer.K0sBinaryPath())
+	dir := h.k0sBinaryPathDir()
 	if err := h.Execf(`install -m 0755 -o root -g root -d "%s"`, dir, exec.Sudo(h)); err != nil {
 		return fmt.Errorf("create k0s binary dir: %w", err)
 	}
@@ -419,7 +425,12 @@ func (h *Host) DrainNode(node *Host) error {
 	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "drain --grace-period=120 --force --timeout=5m --ignore-daemonsets --delete-emptydir-data %s", node.Metadata.Hostname), exec.Sudo(h))
 }
 
-// UncordonNode marks the node schedulable again
+// CordonNode marks the node unschedulable
+func (h *Host) CordonNode(node *Host) error {
+	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "cordon %s", node.Metadata.Hostname), exec.Sudo(h))
+}
+
+// UncordonNode marks the node schedulable
 func (h *Host) UncordonNode(node *Host) error {
 	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "uncordon %s", node.Metadata.Hostname), exec.Sudo(h))
 }
@@ -429,14 +440,6 @@ func (h *Host) DeleteNode(node *Host) error {
 	return h.Exec(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "delete node %s", node.Metadata.Hostname), exec.Sudo(h))
 }
 
-func (h *Host) LeaveEtcd(node *Host) error {
-	etcdAddress := node.SSH.Address
-	if node.PrivateAddress != "" {
-		etcdAddress = node.PrivateAddress
-	}
-	return h.Exec(h.Configurer.K0sCmdf("etcd leave --peer-address %s --datadir %s", etcdAddress, h.K0sDataDir()), exec.Sudo(h))
-}
-
 // CheckHTTPStatus will perform a web request to the url and return an error if the http status is not the expected
 func (h *Host) CheckHTTPStatus(url string, expected ...int) error {
 	status, err := h.Configurer.HTTPStatus(h, url)
